from typing import Dict, Generic, Tuple, TypeVar, List
from lang.cfg import CFG, Node

T = TypeVar("T")


class Analysis(Generic[T]):
    # Abstract Class

    # Subclass this class with type T2 by writing
    #   class SubclassOfAnalysis(Analysis[T2]):
    #       ...

    program: CFG

    def __init__(self, program: CFG) -> None:
        self.program = program

    def initial_in(self, node: Node) -> T:
        raise NotImplementedError

    def initial_out(self, node: Node) -> T:
        raise NotImplementedError

    def get_new_values(self, node: Node, predecessors_out: List[T], successors_in: List[T], curr_out: Dict[Node, T], curr_in: Dict[Node, T]) -> Tuple[T, T]:
        # Return a tuple (In: T, Out: T) where In is the new entry value of node
        # and `Out is the new exit value of node.
        raise NotImplementedError