from analysis.fixed_point import fixed_point
from analysis.definitely_initialized import DefinitelyInitialized
from lang import stringToCFG
from lang.utils import getValueByLabel
from lang.cfg import Variable


def test_definitely_initialized_1():
    s = r"""
        a = 0;
        if (a > 1) {
            x = 0;
        } else {}
        l1: a = a + x;"""
    cfg = stringToCFG(s)
    In, Out = fixed_point(cfg, DefinitelyInitialized(cfg))
    assert ({Variable("a")} == getValueByLabel("l1", Out))


def test_definitely_initialized_2():
    s = r"""
        a = -5;
        if (a < 0) {
            x = 5;
        } else {
            x = 0;
        }
        l1: a = a + x;"""
    cfg = stringToCFG(s)
    In, Out = fixed_point(cfg, DefinitelyInitialized(cfg))
    assert ({Variable("a"), Variable("x")} == getValueByLabel("l1", Out))


def test_definitely_initialized_3():
    s = r"""
            i = 0;
        l1: while (i < n) {
                i = i + 1;
            }"""
    cfg = stringToCFG(s)
    In, Out = fixed_point(cfg, DefinitelyInitialized(cfg))
    # be overconservative with loop headers
    assert ({Variable("i")} == getValueByLabel("l1", Out))


def test_definitely_initialized_4():
    s = r"""
        l1: if (n == 0) {
                n = 10;
            } else {
                n = 2;
            }
        l2: i = 0;
            while (i < n) {
                i = i + 1;
            }"""
    cfg = stringToCFG(s)
    In, Out = fixed_point(cfg, DefinitelyInitialized(cfg))
    assert (set() == getValueByLabel("l1", Out))
    assert ({Variable("i"), Variable("n")} == getValueByLabel("l2", Out))


def test_definitely_initialized_5():
    s = r"""
            n = 10;
        l1: i = 0;
            while (i < n) {
                i = i + 1;
            }"""
    cfg = stringToCFG(s)
    In, Out = fixed_point(cfg, DefinitelyInitialized(cfg))
    assert ({Variable("i"), Variable("n")} == getValueByLabel("l1", Out))


def test_definitely_initialized_6():
    s = r"""
            n = 10;
        l1: while (i < n) {
                i = i + 1;
            }"""
    cfg = stringToCFG(s)
    In, Out = fixed_point(cfg, DefinitelyInitialized(cfg))
    assert (set() == getValueByLabel("l1", In))
