from lang import stringToCFG
from lang.ast.ast import BinOp, BinaryExpression, BooleanConstant, IntegerConstant, UnOp, UnaryExpression, Variable
from lang.utils import getExpressionsInCFG


def test_get_expressions_in_cfg1():
    s = """
    x = 1; 
    y = 2;
    """
    cfg = stringToCFG(s)
    assert(getExpressionsInCFG(cfg, include_atomic=True) == {
           IntegerConstant(1), IntegerConstant(2)})
    assert(getExpressionsInCFG(cfg, include_atomic=False) == set())


def test_get_expressions_in_cfg2():
    s = """
    while (true) { }
    """
    cfg = stringToCFG(s)
    assert(getExpressionsInCFG(cfg, include_atomic=True) == {
           BooleanConstant(True)})
    assert(getExpressionsInCFG(cfg, include_atomic=False) == set())


def test_get_expressions_in_cfg3():
    s = """
    while (true == false) { }
    """
    cfg = stringToCFG(s)
    assert(getExpressionsInCFG(cfg, include_atomic=True) == {
           BooleanConstant(True), BooleanConstant(False), BinaryExpression(BooleanConstant(True), BinOp.Equals, BooleanConstant(False))})
    assert(getExpressionsInCFG(cfg, include_atomic=False) == {
        BinaryExpression(BooleanConstant(True), BinOp.Equals, BooleanConstant(False))})


def test_get_expressions_in_cfg4():
    s = """
    if ((1 + 2) == (1 + 2)) { }
    else { 
        x = 1 + 2; 
    }
    """
    cfg = stringToCFG(s)
    assert(getExpressionsInCFG(cfg, include_atomic=True) == {
           IntegerConstant(1), IntegerConstant(2), BinaryExpression(IntegerConstant(1), BinOp.Plus, IntegerConstant(2)), BinaryExpression(BinaryExpression(IntegerConstant(1), BinOp.Plus, IntegerConstant(2)), BinOp.Equals, BinaryExpression(IntegerConstant(1), BinOp.Plus, IntegerConstant(2)))})
    assert(getExpressionsInCFG(cfg, include_atomic=False) == {
        BinaryExpression(IntegerConstant(1), BinOp.Plus, IntegerConstant(2)), BinaryExpression(BinaryExpression(IntegerConstant(1), BinOp.Plus, IntegerConstant(2)), BinOp.Equals, BinaryExpression(IntegerConstant(1), BinOp.Plus, IntegerConstant(2)))})


def test_get_expressions_in_cfg5():
    s = """
    x = 1 + x; 
    y = 2;
    """
    cfg = stringToCFG(s)
    assert(getExpressionsInCFG(cfg, include_atomic=True) == {IntegerConstant(1), IntegerConstant(
        2), Variable("x"), BinaryExpression(IntegerConstant(1), BinOp.Plus, Variable("x"))})
    assert(getExpressionsInCFG(cfg, include_atomic=False) == {
           BinaryExpression(IntegerConstant(1), BinOp.Plus, Variable("x"))})


def test_get_expressions_in_cfg6():
    s = """
    if (x > 2) {
        x = 1; 
    } else {
        z = x + (1 * 2);
    }
    z = 2;
    """
    cfg = stringToCFG(s)
    assert(getExpressionsInCFG(cfg, include_atomic=True) == {IntegerConstant(1), IntegerConstant(
        2), Variable("x"), BinaryExpression(IntegerConstant(1), BinOp.Times, IntegerConstant(2)), BinaryExpression(Variable("x"), BinOp.Plus, BinaryExpression(IntegerConstant(1), BinOp.Times, IntegerConstant(2))), BinaryExpression(Variable("x"), BinOp.GreaterThan, IntegerConstant(2))})
    assert(getExpressionsInCFG(cfg, include_atomic=False) == {BinaryExpression(IntegerConstant(1), BinOp.Times, IntegerConstant(2)), BinaryExpression(Variable(
        "x"), BinOp.Plus, BinaryExpression(IntegerConstant(1), BinOp.Times, IntegerConstant(2))), BinaryExpression(Variable("x"), BinOp.GreaterThan, IntegerConstant(2))})


def test_get_expressions_cfg7():
    s = """
    if (x > (-2)) {
         
    } else {
    }
    z = -3;
    """
    cfg = stringToCFG(s)
    assert(getExpressionsInCFG(cfg) == {UnaryExpression(
        UnOp.Minus, IntegerConstant(2)), UnaryExpression(UnOp.Minus, IntegerConstant(3)), BinaryExpression(Variable("x"), BinOp.GreaterThan, UnaryExpression(UnOp.Minus, IntegerConstant(2)))})
