from analysis.fixed_point import fixed_point
from analysis.partially_available_expressions import PartiallyAvailableExpressions
from lang.ast.ast import Assignment, BinOp, BinaryExpression, IntegerConstant, Variable
from lang.to_cfg import stringToCFG
from lang.utils import getEntryValue, getExitValue, getExpressionsInCFG, getNodeByLabel, getValueByLabel
from lang.cfg import InnerNode


def test_available_expressions_sequential1():
    s = """
    l1: x = 0 + 1; 
    """
    cfg = stringToCFG(s)
    In, Out = fixed_point(cfg, PartiallyAvailableExpressions(cfg))
    expr = BinaryExpression(IntegerConstant(0), BinOp.Plus, IntegerConstant(1))
    assert(getEntryValue(Out) == set())
    assert(getExitValue(In) == {expr})
    assert(getValueByLabel("l1", In) == set())
    assert(getValueByLabel("l1", Out) == {expr})


def test_available_expression_loop1():
    s = """
            x = a + b;
        l1: while (x < 0) {
        l2:     a = a + b;
            }
    """
    cfg = stringToCFG(s)
    In, Out = fixed_point(cfg, PartiallyAvailableExpressions(cfg))
    assert(BinaryExpression(Variable("a"), BinOp.Plus,
           Variable("b")) in getValueByLabel("l1", In))
    assert(BinaryExpression(Variable("a"), BinOp.Plus,
           Variable("b")) in getValueByLabel("l2", In))
    assert(BinaryExpression(Variable("a"), BinOp.Plus, Variable("b"))
           not in getValueByLabel("l2", Out))

