from analysis.fixed_point import fixed_point
from analysis.potential_placement import PotentialPlacement, delete, insert
from lang.ast.ast import BinOp, BinaryExpression, IntegerConstant, Variable
from lang.cfg import Entry
from lang.to_cfg import stringToCFG
from lang.utils import getNodeByLabel

def test_potential_placement_misc3():
    s = """
    l1: while (x < 0) {
            y = a + b;
        }
    """
    cfg = stringToCFG(s)
    In, Out = fixed_point(cfg, PotentialPlacement(cfg))
    a_plus_b_expr = BinaryExpression(Variable("a"), BinOp.Plus, Variable("b"))
    x_lt_expr = BinaryExpression(Variable("x"), BinOp.LessThan, IntegerConstant(0))

    for node in cfg.nodes: 
        if isinstance(node, Entry):
            assert({x_lt_expr} == insert(cfg)[node])
        else: 
            assert(set() == insert(cfg)[node])
    
    for node in cfg.nodes: 
        if node == getNodeByLabel("l1", cfg.nodes):
            assert({x_lt_expr} == delete(cfg)[node])
        else: 
            assert(set() == delete(cfg)[node])

def test_potential_placement_misc4():
    s = """
    l1: while (x < 0) {
    l2:     y = a + b;
        }
    l3: z = a + b;
    """
    cfg = stringToCFG(s)
    In, Out = fixed_point(cfg, PotentialPlacement(cfg))
    a_plus_b_expr = BinaryExpression(Variable("a"), BinOp.Plus, Variable("b"))
    x_lt_expr = BinaryExpression(Variable("x"), BinOp.LessThan, IntegerConstant(0))

    for node in cfg.nodes: 
        if isinstance(node, Entry):
            assert({x_lt_expr, a_plus_b_expr} == insert(cfg)[node])
        else: 
            assert(set() == insert(cfg)[node])
    
    for node in cfg.nodes: 
        if node == getNodeByLabel("l1", cfg.nodes):
            assert({x_lt_expr} == delete(cfg)[node])
        elif node in { getNodeByLabel("l2", cfg.nodes), getNodeByLabel("l3", cfg.nodes)}: 
            assert({a_plus_b_expr} == delete(cfg)[node])
        else: 
            assert(set() == delete(cfg)[node])
