from analysis.fixed_point import fixed_point
from analysis.sign_analysis import SignAnalysis, Lattice
from lang import stringToCFG
from lang.utils import getValueByLabel
from lang.cfg import Variable

def test_sign_analysis_1():
    s = r"""
        x = 0;
        y = x + 2;
    l1: z = x*y;
    """
    cfg = stringToCFG(s)
    In, Out = fixed_point(cfg, SignAnalysis(cfg))
    assert (Lattice.Zero == getValueByLabel("l1", Out)[Variable("z")])


def test_sign_analysis_2():
    s = r"""
        x = 1;
    l1: y = -x;
    """
    cfg = stringToCFG(s)
    In, Out = fixed_point(cfg, SignAnalysis(cfg))
    assert (Lattice.Negative == getValueByLabel("l1", Out)[Variable("y")])


def test_sign_analysis_3():
    s = r"""
        if (x == 2) {
          a = 1;
        } else {
          a = -1;
        }
    l1: b = a*1;
    """
    cfg = stringToCFG(s)
    In, Out = fixed_point(cfg, SignAnalysis(cfg))
    assert (Lattice.NonZero == getValueByLabel("l1", Out)[Variable("b")])

def test_sign_analysis_4():
    s = r"""
        x = 0;
        y = 0;
    l1: z = (x + y)*x;
    """
    cfg = stringToCFG(s)
    In, Out = fixed_point(cfg, SignAnalysis(cfg))
    assert (Lattice.Zero == getValueByLabel("l1", Out)[Variable("z")])